#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# =======================================================


"""engine_bing
    * Bing用の検索用Classを持つモジュール.
"""


import datetime
import json
import re

from urllib import parse
from bs4 import BeautifulSoup

from .common import Color
from .engine_common import CommonEngine


class Bing(CommonEngine):
    """Bing

    Bing用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Bing'
        self.COLOR = Color.CYAN
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.bing.com/'
        self.SEARCH_URL = 'https://www.bing.com/search'
        self.IMAGE_URL = 'https://www.bing.com/images/async'
        self.SUGGEST_URL = 'https://www.bing.com/AS/Suggestions'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        search_url = ''

        # 検索タイプがtextの場合
        if type == 'text':
            # 検索urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,    # 検索キーワード
                'count': '100',  # 1ページごとの表示件数
                'go': '検索',
                'qs': 'ds',
                'from': 'QBRE',
                'filters': '',   # 期間含めフィルターとして指定するパラメータ
                'first': ''      # 開始位置
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['mkt'] = self.LANG + '-' + self.LOCALE

            # rangeが設定されている場合
            try:
                start = self.RANGE_START
                end = self.RANGE_END

                unix_day = datetime.strptime('1970-01-01', "%Y-%m-%d")
                cd_min = (start - unix_day).days
                cd_max = (end - unix_day).days

                # GETパラメータに日時データを追加
                url_param['filters'] = 'ex1:"ez5_{0}_{1}"'.format(
                    cd_min, cd_max)

            except AttributeError:
                None

        # 検索タイプがimageの場合
        elif type == 'image':
            # 検索urlを指定
            search_url = self.IMAGE_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,    # 検索キーワード
                'count': '100',  # 1回ごとの件数
                'first': '',     # 検索位置
                'tsc': 'ImageBasicHover',
                'layout': 'RowBased',
            }

            # rangeが指定されている場合
            # TODO: 日時パラメータを追加(ex: `qft=+filterui%3aage-lt43200`)

        page = 0
        while True:
            # parameterにページを開始する番号を指定
            url_param['first'] = str(page * 100)
            params = parse.urlencode(url_param)

            target_url = search_url + '?' + params

            yield 'GET', target_url, None

            page += 1

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        url_param = {
            'qry': keyword,  # 検索キーワード
            'cvid': 'F5F47E4155E44D86A86690B49023B0EF'
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        if type == 'text':
            self.SOUP_SELECT_URL = 'h2 > a'
            self.SOUP_SELECT_TITLE = 'h2 > a'

        elif type == 'image':
            self.SOUP_SELECT_URL = '.imgpt > .iusc'

        # CommonEngineの処理を呼び出す
        links = super().get_links(html, type)

        return links

    # 画像検索ページの検索結果(links(list()))を生成するfunction
    def get_image_links(self, soup: BeautifulSoup):
        """get_image_links
        BeautifulSoupから画像検索ページを解析して結果を返す関数.

        Args:
            soup (BeautifulSoup): 解析するBeautifulSoupオブジェクト.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)
        """

        elements = soup.select(self.SOUP_SELECT_URL)
        edata = [e['m'] for e in elements]

        result = []  # image url
        for e in edata:
            # json化
            je = json.loads(e)

            etitle = je['t']
            elink = je['purl']
            eimage = je['murl']

            el = {
                'title': etitle,
                'pagelink': elink,
                'link': eimage,
            }

            result.append(el)

        return result

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """
        soup = BeautifulSoup(html, 'lxml')
        elements = soup.select('ul > li')
        suggests[char if char == '' else char[-1]] = [e['query']
                                                      for e in elements]
        return suggests

    def processings_elist(self, elinks: list, etitles: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
        """
        new_elinks = list()
        new_etitles = list()

        # `https://rd.listing.yahoo.co.jp/`を除外する
        yahoo_match = re.compile(r'^https://rd\.listing\.yahoo\.co\.jp/')

        n = 0
        for link in elinks:
            if link[0] != '/' and not yahoo_match.match(link):
                new_elinks.append(link)
                if len(etitles) > n:
                    new_etitles.append(etitles[n])
            n += 1

        return new_elinks, new_etitles



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# =======================================================

"""subcommand
    * pydorkをコマンドとして動作させる際の処理を記載しているモジュール
"""


import sys
import threading
import json

from .engine import SearchEngine, ENGINES
from .common import Color
from .common import Message


# サブコマンドの動作集約用関数
def run_subcommand(subcommand, args):
    """run_subcommand

    Args:
        subcommand (str): 使用するサブコマンド([search, suggest]).
        args (Namespace): argparseで取得した引数(Namespace).
    """

    target = None
    search_mode = ''
    if subcommand == 'search':
        # チェック処理
        if ((args.start is None and args.end is not None) or (args.start is not None and args.end is None)):
            print(
                Color.GRAY + "期間を指定する場合は--start, --endの両方を指定してください" + Color.END,
                file=sys.stderr
            )
            return
        target = search
        search_mode = 'text'

    elif subcommand == 'image':
        target = search
        search_mode = 'image'

    elif subcommand == 'suggest':
        target = suggest

    tasks = []
    lock = threading.Lock()
    for st in args.search_type:
        # if all
        if st == 'all':
            for engine in ENGINES:
                task = threading.Thread(
                    target=target, args=(engine, args, True, lock, search_mode))
                tasks.append(task)

            continue

        # if in searchengine
        if st in ENGINES:
            task = threading.Thread(
                target=target, args=(st, args, True, lock, search_mode))
            tasks.append(task)

            continue

    for task in tasks:
        task.start()

    for task in tasks:
        task.join()


# SearchEngineのオプション設定用関数
def set_se_options(se, args):
    """set_se_options

    Args:
        se (SearchEngine): argsの情報を元に、オプションを設定するSearchEngine.
        args (Namespace): argparseで取得した引数(Namespace).

    Returns:
        SearchEngine: オプションを設定したSearchEngine.
    """

    # set debug flag
    if 'debug' in args:
        se.set_is_debug(args.debug)

    # set is_command flag
    se.set_is_command(True)

    # set disable headless
    if 'disable_headless' in args:
        se.set_disable_headless(args.disable_headless)

    # proxy
    if args.proxy != '':
        se.set_proxy(args.proxy)

    # Selenium
    if args.selenium:
        # set default endpoint
        endpoint = None

        # if set browser-endpoint
        if args.browser_endpoint != "":
            endpoint = args.browser_endpoint

        # set selenium
        se.set_selenium(endpoint, args.browser)

    # Splush
    if args.splash:
        # set default endpoint
        endpoint = 'localhost:8050'

        # if set browser-endpoint
        if args.browser_endpoint != "":
            endpoint = args.browser_endpoint

        # set splash
        se.set_splash(endpoint)

    # useragent
    se.set_user_agent()

    # lang/country code
    se.set_lang(args.lang, args.country)

    # set cookie driver(last set)
    se.set_cookie_files(args.cookies)

    return se


# 検索
def search(engine, args, cmd=False, lock=None, mode='text'):
    """search

    Args:
        engine (str): 使用する検索エンジン(.engine.ENGINES).
        args (Namespace): argparseで取得した引数(Namespace).
        cmd (bool, optional): commandで実行しているか否か. Defaults to False.
        lock (threading.Lock): threadingのマルチスレッドで使用するLock.現在は未使用. Defaults to None.
        type (str, optional): 検索タイプ. `text` or `image`.
    """

    # start search engine class
    se = SearchEngine()

    # Set Engine
    se.set(engine)

    # Set SearchEngine options
    se = set_se_options(se, args)

    # Set lock
    se.set_lock(lock)

    # Set color
    if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
        se.set_is_color(True)

    # 検索タイプを設定(テキスト or 画像)
    search_type = mode

    # 検索を実行
    result = se.search(
        args.query, type=search_type,
        maximum=args.num
    )

    # sep
    sep = ': '
    if args.nullchar:
        sep = '\0'

    # debug
    se.ENGINE.MESSAGE.print_text(
        json.dumps(result),
        separator=sep,
        header=se.ENGINE.MESSAGE.HEADER + ': ' +
        Color.GRAY + '[DEBUG]: [Result]' + Color.END,
        mode="debug",
    )

    # title出力を行うか確認
    title_mode = False
    if 'title' in args:
        title_mode = args.title

    # pageurl出力を行うか確認
    pagelink_mode = False
    if 'pagelink' in args:
        pagelink_mode = args.pagelink

    # 検索結果を出力
    for d in result:
        data = []
        link = d['link']

        # 出力dataにlinkを追加
        data.insert(0, link)

        # pageurlの有無を確認
        if 'pagelink' in d and pagelink_mode:
            pagelink = d['pagelink']

            # pagelinkの色指定
            if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
                pagelink = Color.GRAY + Color.UNDERLINE + pagelink + Color.END

            data.insert(0, pagelink)

        # titleの有無を確認
        if 'title' in d and title_mode:
            title = d['title']

            # titleの色指定
            if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
                title = Color.GRAY + title + Color.END

            data.insert(0, title)

        se.ENGINE.MESSAGE.print_line(*data, separator=sep)


# サジェスト
def suggest(engine, args, cmd=False, lock=None, mode=''):
    """suggest

    Args:
        engine (str): 使用する検索エンジン(.engine.ENGINES).
        args (Namespace): argparseで取得した引数(Namespace).
        cmd (bool, optional): commandで実行しているか否か. Defaults to False.
        lock (threading.Lock): threadingのマルチスレッドで使用するLock.現在は未使用. Defaults to None.
        mode (str, optional): マルチスレッドでsearchとある程度共用で使えるようにするための引数. 利用していない. Defaults to ''.
    """

    # start search engine class
    se = SearchEngine()

    # Set Engine
    se.set(engine)

    # Set Message()
    msg = Message()
    msg.set_engine(se.ENGINE.NAME, se.ENGINE.COLOR)
    if 'debug' in args:
        msg.set_is_debug(args.debug)
    msg.set_is_command(True)

    # set msg to se
    se.ENGINE.set_messages(msg)

    # Set SearchEngine options
    se = set_se_options(se, args)

    # Set lock
    se.set_lock(lock)

    # Header
    header = '[${ENGINE_NAME}Suggest]'
    if args.color == 'always' or (args.color == 'auto' and sys.stdout.isatty()):
        sc = Color(se.ENGINE.COLOR)
        header = sc.out(header)
    se.ENGINE.MESSAGE.set_header(header)

    # Suggestを取得
    result = se.suggest(
        args.query,
        jap=args.jap,
        alph=args.alph,
        num=args.num,
    )

    for words in result.values():
        for w in words:
            se.ENGINE.MESSAGE.print_line(w, separator=": ")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# =======================================================


"""engine_google
    * Google用の検索用Classを持つモジュール.
"""


import json
import os

from time import sleep
from json.decoder import JSONDecodeError
from urllib import parse
from lxml import etree

from .common import Color
from .recaptcha import TwoCaptcha
from .engine_common import CommonEngine


# Google画像検索で使用するパラメータID
RPC_ID = "HoAMBc"


class Google(CommonEngine):
    """Google

    Google用の検索エンジン用Class.
    """

    def __init__(self):
        # CommonEngineの処理を呼出し
        super().__init__()

        self.NAME = 'Google'
        self.COLOR = Color.PURPLE
        self.COLOR_NAME = self.COLOR + self.NAME + Color.END

        # リクエスト先のURLを指定
        self.ENGINE_TOP_URL = 'https://www.google.com/'
        self.SEARCH_URL = 'https://www.google.com/search'
        self.IMAGE_URL = 'https://www.google.com/_/VisualFrontendUi/data/batchexecute'
        self.SUGGEST_URL = 'http://www.google.com/complete/search'

        # ReCaptcha画面かどうかの識別用
        self.SOUP_RECAPTCHA_TAG = '#captcha-form > #recaptcha'

    def gen_search_url(self, keyword: str, type: str):
        """gen_search_url

        検索用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.
            type (str): 検索タイプ.

        Returns:
            dict: 検索用url
        """

        search_url = ''

        if type == 'text':
            # 検索用urlを指定
            search_url = self.SEARCH_URL

            # 検索パラメータの設定
            url_param = {
                'q': keyword,   # 検索キーワード
                'oq': keyword,  # 検索キーワード
                'num': '100',   # 1ページごとの表示件数
                'filter': '0',  # 類似ページのフィルタリング(0...無効, 1...有効)
                'start': '',    # 開始位置
                'tbs': '',      # 期間
                'nfpr': '1'     # もしかして検索(Escape hatch)を無効化
            }

            # lang/localeが設定されている場合
            if self.LANG != '' and self.LOCALE != '':
                url_param['hl'] = self.LANG
                url_param['gl'] = self.LOCALE

            # rangeが設定されている場合
            try:
                start = self.RANGE_START
                end = self.RANGE_END

                cd_min = start.strftime("%m/%d/%Y")
                cd_max = end.strftime("%m/%d/%Y")

                # GETパラメータに日時データを追加
                url_param['tbs'] = "cdr:1,cd_min:{0},cd_max:{1}".format(
                    cd_min, cd_max)

            except AttributeError:
                None

            page = 0
            while True:
                # parameterにページを開始する番号を指定
                url_param['start'] = str(page * 100)
                params = parse.urlencode(url_param)

                target_url = search_url + '?' + params

                yield 'GET', target_url, None

                page += 1

        elif type == 'image':
            # 検索用urlを指定
            search_url = self.IMAGE_URL

            # Refererの設定
            if not self.USE_SELENIUM:
                self.session.headers.update(
                    {"Referer": "https://www.google.com/"}
                )

            # 検索パラメータの設定
            url_param = {
                'rpcids': 'HoAMBc',
                'hl': 'id',
                'authuser': '0',
                'soc-app': '162',
                'soc-platform': '1',
                'soc-device': '1',
                'rt': 'c'
            }

            # 画像のカーソル位置指定パラメータを作成
            self.image_next_cursor = None
            self.image_cursor = []

            page = 0
            while True:
                # post dataを生成
                data = {
                    "f.req": build_rpc_request(keyword, (self.image_cursor, self.image_next_cursor), page),
                    "at": "ABrGKkQnVYg89U_cdKuhNZ5hM4vx:1616119655028",
                    # "": "",
                }

                params = parse.urlencode(url_param)
                target_url = search_url + '?' + params

                yield 'POST', target_url, data

    def gen_suggest_url(self, keyword: str):
        """gen_suggest_url

        サジェスト取得用のurlを生成する.

        Args:
            keyword (str): 検索クエリ.

        Returns:
            dict: サジェスト取得用url
        """

        url_param = {
            'q': keyword,  # 検索キーワード
            'output': 'toolbar',
            'ie': 'utf-8',
            'oe': 'utf-8',
        }

        params = parse.urlencode(url_param)
        url = self.SUGGEST_URL + '?' + params

        return url

    def get_links(self, html: str, type: str):
        """get_links

        受け付けたhtmlを解析し、検索結果をlistに加工して返す関数.

        Args:
            html (str): 解析する検索結果のhtml.
            type (str): 検索タイプ([text, image]).現時点ではtextのみ対応.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'url': 'https://hogehoge....'}, {...}]`)
        """

        # テキスト検索の場合
        if type == 'text':
            # request or seleniumの定義
            self.SOUP_SELECT_URL = '#main > div > div > .kCrYT > a'
            self.SOUP_SELECT_TITLE = '#main > div > div > .kCrYT > a > h3 > div'

            # Selenium経由、かつFirefoxを使っている場合
            if self.USE_SELENIUM and self.SELENIUM_BROWSER == 'firefox':
                self.SOUP_SELECT_URL = '.yuRUbf > a'
                self.SOUP_SELECT_TITLE = '.yuRUbf > a > .LC20lb'

            # Splash経由で通信している場合
            elif self.USE_SPLASH:
                self.SOUP_SELECT_URL = '.tF2Cxc > .yuRUbf > a'
                self.SOUP_SELECT_TITLE = '.tF2Cxc > .yuRUbf > a > .LC20lb'

            # CommonEngineの処理を呼び出す
            links = super().get_links(html, type)

        # イメージ検索の場合
        elif type == 'image':
            links = self.get_image_links(html)

        return links

    # 画像検索ページの検索結果(links(list()))を生成するfunction
    def get_image_links(self, html: str):
        """get_image_links

        BeautifulSoupから画像検索ページを解析して結果を返す関数.
        Seleniumを利用し、自動的にページ末尾まで移動して続きを取得する.
        クリック等が発生するため、抽出にかなり時間がかかる.

        Args:
            html (str): 解析する検索結果のhtml.

        Returns:
            list: 検索結果(`[{'title': 'title...', 'link': 'https://hogehoge....'}, {...}]`)

        参考:
            - https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/client.py
        """

        links = list()

        # 改行区切りでloop
        for line in html.split("\n"):
            if RPC_ID not in line:
                continue

            # Make it json readable
            line_cl = line.replace("\\n", "")  # Remove \n

        lineson = json.loads(line_cl)

        data = pjson_loads(lineson[0][2])

        # 画像のカーソル位置を更新
        self.image_next_cursor = data[-2]
        self.image_img_cursor = data[31][0][12][11][5]

        for img in data[31][0][12][2]:
            # imgの値をチェック
            if img[1] is None:
                continue

            link = img[1][3][0]  # 画像ファイルのurl
            title = img[1][9]['2003'][3]  # 画像ファイルのあるページのtitle
            pagelink = img[1][9]['2003'][2]  # 画像ファイルのあるページのurl
            links.append(
                {
                    "link": link,
                    "title": title,
                    "pagelink": pagelink,
                }
            )

        return links

    def get_suggest_list(self, suggests: list, char: str, html: str):
        """get_suggest_list

        htmlからsuggestを配列で取得する関数.

        Args:
            suggests (list): suggestを追加するための大本のlist.
            char (str): サジェストの文字列.
            html (str): 解析を行うhtml.

        Returns:
            dict: サジェスト配列
        """

        sug_root = etree.XML(html)
        sug_data = sug_root.xpath("//suggestion")
        data = [s.get("data") for s in sug_data]

        suggests[char if char == '' else char[-1]] = data

        return suggests

    def processings_elist(self, elinks: list, etitles: list):
        """processings_elist

        self.get_links 内で、取得直後のelinks, etitlesに加工を加えるための関数.

        Args:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列

        Returns:
            elinks (list): elinks(検索結果のlink)の配列
            etitles (list): etitles(検索結果のtitle)の配列
        """

        # seleniumでfirefoxを使っていない、かつsplashを使っていない場合
        new_elinks = []
        for elink in elinks:
            parsed = parse.urlparse(elink)
            parsed_query = parse.parse_qs(parsed.query)

            if 'url' in parsed_query and elink[0] == '/':
                parsed_q = parsed_query['url']
                if len(parsed_q) > 0:
                    new_elink = parsed_q[0]
                    new_elinks.append(new_elink)
            else:
                new_elinks.append(elink)
        elinks = list(dict.fromkeys(new_elinks))

        return elinks, etitles

    def bypass_recaptcha_selenium(self, url: str, html: str):
        """bypass_recaptcha_selenium

        SeleniumでReCaptchaを突破する関数.
        2Captchaでの自動突破の場合、CookieとProxyが必要となる.

        Args:
            url (str): ReCaptcha画面が表示されてしまったリクエストのurl
            html (str): ReCaptcha画面のhtml

        Returns:
            str: ReCaptchaを突破後のurlのhtml
        """

        # resultを定義しておく
        result = None

        # 環境変数を取得
        TC_API_KEY = os.getenv('API_KEY_2CAPTCHA')

        # Seleniumの場合、手動でBypassが行えるようにする
        if self.IS_DISABLE_HEADLESS:
            while True:
                # 現在のSeleniumのurlを取得する
                current_url = self.driver.current_url
                current_url_parse = parse.urlparse(current_url)

                # current_urlのpathが `/sorry/index` か識別する
                current_url_path = current_url_parse.path
                if current_url_path != '/sorry/index':
                    break

                # 待機
                sleep(1)

            sleep(5)

            # 現在のページ(ReCaptchaから移動したページ)のhtmlを取得する
            result = self.driver.page_source

        # self.IS_DISABLE_HEADLESS がFalseで、かつ`API_KEY_2CAPTCHA`が定義されている場合
        elif TC_API_KEY is not None:
            # solverを作成
            solver = TwoCaptcha(TC_API_KEY)

            # flag set
            solver.set_debug(self.IS_DEBUG)
            solver.set_command(self.IS_COMMAND)
            solver.set_user_agent(self.USER_AGENT)
            solver.set_messages(self.MESSAGE)

            # solverからのレスポンスを取得する
            code = solver.google_recaptcha(
                html=html,
                url=url,
                cookies=self.driver.get_cookies(),
                proxy=self.PROXY,

            )

            # ReCaptchaの解除に失敗した場合
            if code is None:
                return result

            # 解除コードを所定のtextareaに入力
            self.driver.execute_script("""
              document.getElementById(
                  "g-recaptcha-response").innerHTML = arguments[0]
            """, code)

            # ボタンクリック
            self.driver.execute_script(
                'var element=document.getElementById("g-recaptcha-response"); element.style.display="none";')

            self.driver.execute_script('submitCallback()')

            sleep(10)

            # 結果を取得する
            result = self.driver.page_source

        return result


def build_rpc_request(keyword: str, cursor: list, page: int):
    """build_rpc_request

    画像検索で使用するrpcデータの生成用関数.

    Original:
        https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/utils.py

    Args:
        keyword (str): [description]
        cursor (list): [description]
        page (int): [description]

    Returns:
        [type]: [description]
    """

    RPC_ID = "HoAMBc"

    return json.dumps(
        [
            [
                [
                    RPC_ID,
                    json.dumps(
                        [
                            None,
                            None,
                            [
                                1,
                                None,
                                450,
                                1,
                                1280,
                                cursor[0],
                                [],
                                [],
                                None,
                                None,
                                None,
                                0,
                                310,
                                [],
                            ],
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            [
                                keyword,
                                None,
                                None,
                                "strict",
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                None,
                                "lnms",
                            ],
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            None,
                            [
                                cursor[1],
                                "CAM=",
                                "CgtHUklEX1NUQVRFMBAaIAA=",
                            ],
                        ],
                        separators=(",", ":"),
                    ),
                    None,
                    "generic",
                ],
            ]
        ],
        separators=(",", ":"),
    )


def pjson_loads(text):
    """pjson_loads

    画像検索で使用するデータの生成用関数.

    Original:
        https://github.com/Wikidepia/py-googleimages/blob/b781b79e9bf40d29cf6fcbdcf625303abf3718bd/googleimages/utils.py

    Args:
        text ([type]): [description]

    Returns:
        [type]: [description]
    """
    while True:
        try:
            data = json.loads(text, strict=False)
        except JSONDecodeError as exc:
            if exc.msg == "Invalid \\escape":
                text = text[: exc.pos] + "\\" + text[exc.pos:]
            else:
                raise
        else:
            return data



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# =======================================================


import argparse
import copy

from datetime import datetime
from pkg_resources import get_distribution

from .engine import ENGINES
from .subcommands import run_subcommand


# version (setup.pyから取得してくる)
__version__ = get_distribution('pydork').version


# main
def main():
    # parserの作成
    parser = argparse.ArgumentParser(
        description='各種検索エンジンから指定したクエリの結果(url)およびSuggestを取得するスクリプト')
    subparsers = parser.add_subparsers()

    # ENGINESに`all`を追加
    engines_list = copy.deepcopy(ENGINES)
    engines_list.append('all')

    # サブコマンド共通の引数
    common_args_map = [
        {
            "args": ["query"],
            "action": "store",
            "type": str,
            "help": "検索文字列(クエリ)",
        },
        {
            "args": ["-t", "--search_type"],
            "default": ["google"],
            "choices": engines_list,
            "nargs": "+",
            "type": str,
            "help": "使用する検索エンジンを指定",
        },
        {
            "args": ["-l", "--lang"],
            "default": "ja",
            "choices": ["ja", "en"],
            "type": str,
            "help": "言語を指定",
        },
        {
            "args": ["-c", "--country"],
            "default": "JP",
            "choices": ["JP", "US"],
            "type": str,
            "help": "言語を指定",
        },
        {
            "args": ["-P", "--proxy"],
            "default": "",
            "type": str,
            "help": "プロキシサーバーを指定(例:socks5://hogehoge:8080, https://fugafuga:18080)",
        },
        {
            "args": ["-s", "--selenium"],
            "action": "store_true",
            "help": "Selenium(headless browser)を使用する(排他: Splashより優先)",
        },
        {
            "args": ["-S", "--splash"],
            "action": "store_true",
            "help": "Splash(headless browser)を使用する(排他: Seleniumの方が優先)",
        },
        {
            "args": ["-b", "--browser-endpoint"],
            "default": "",
            "type": str,
            "help": "Selenium/Splash等のヘッドレスブラウザのエンドポイントを指定(例: localhost:8050)",
        },
        {
            "args": ["-B", "--browser"],
            "default": "firefox",
            "choices": ["chrome", "firefox"],
            "type": str,
            "help": "Seleniumで使用するBrowserを指定",
        },
        {
            "args": ["--color"],
            "default": "auto",
            "choices": ["auto", "none", "always"],
            "type": str,
            "help": "color出力の切り替え"
        },
        {
            "args": ["--cookies"],
            "default": "~/.pydork_cookies",
            "type": str,
            "help": "使用するcookieファイルの格納先ディレクトリのPATH(各検索エンジンごとでcookieファイルを個別保存)"
        },
    ]

    # サブコマンド `search` の引数
    search_args_map = [
        {
            "args": ["-T", "--title"],
            "action": "store_true",
            "help": "検索結果のタイトルをセットで出力する",
        },
        {
            "args": ["-0", "--nullchar"],
            "action": "store_true",
            "help": "null characterを区切り文字として使用する",
        },
        {
            "args": ["-n", "--num"],
            "default": 300,
            "type": int,
            "help": "検索結果の取得数を指定する",
        },
        {
            "args": ["--start"],
            "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
            "help": "期間指定(開始)",
        },
        {
            "args": ["--end"],
            "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
            "help": "期間指定(終了)",
        },
        {
            "args": ["--debug"],
            "action": "store_true",
            "help": "debugモードを有効にする",
        },
        {
            "args": ["--disable-headless"],
            "action": "store_true",
            "help": "Seleniumでheadlessモードを無効化する(手動でのReCaptcha対応時に必要)",
        },
    ]
    search_args_map.extend(copy.deepcopy(common_args_map))

    # サブコマンド `image` の引数
    image_args_map = [
        {
            "args": ["-T", "--title"],
            "action": "store_true",
            "help": "検索結果のタイトルをセットで出力する",
        },
        {
            "args": ["-p", "--pagelink"],
            "action": "store_true",
            "help": "画像ファイルがあるhtmlのURLも出力する",
        },
        {
            "args": ["-0", "--nullchar"],
            "action": "store_true",
            "help": "null characterを区切り文字として使用する",
        },
        {
            "args": ["-n", "--num"],
            "default": 300,
            "type": int,
            "help": "検索結果の取得数を指定する",
        },
        # {
        #     "args": ["--start"],
        #     "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
        #     "help": "期間指定(開始)",
        # },
        # {
        #     "args": ["--end"],
        #     "type": lambda s: datetime.strptime(s, '%Y-%m-%d'),
        #     "help": "期間指定(終了)",
        # },
        {
            "args": ["--debug"],
            "action": "store_true",
            "help": "debugモードを有効にする",
        },
        {
            "args": ["--disable-headless"],
            "action": "store_true",
            "help": "Seleniumでheadlessモードを無効化する(手動でのReCaptcha対応時に必要)",
        },
    ]
    image_args_map.extend(copy.deepcopy(common_args_map))

    # サブコマンド `suggest` の引数
    suggest_args_map = [
        {
            "args": ["--jap"],
            "action": "store_true",
            "help": "日本語の文字を検索キーワードに追加してサジェストを取得"
        },
        {
            "args": ["--alph"],
            "action": "store_true",
            "help": "アルファベット文字を検索キーワードに追加してサジェストを取得"
        },
        {
            "args": ["--num"],
            "action": "store_true",
            "help": "数字を検索キーワードに追加してサジェストを取得"
        },
    ]
    suggest_args_map.extend(copy.deepcopy(common_args_map))

    # search
    # ----------
    parser_search = subparsers.add_parser(
        'search',
        help='search mode. see `search -h`'
    )

    # add_argument
    for element in search_args_map:
        args = element['args']
        element.pop('args')
        parser_search.add_argument(*args, **element)

    # set parser_search
    parser_search.set_defaults(handler=run_subcommand, subcommand="search")

    # image
    # ----------
    # TODO: image検索をサブコマンドとして追加する
    parser_image = subparsers.add_parser(
        'image',
        help='search mode. see `search -h`'
    )

    # add_argument
    for element in image_args_map:
        args = element['args']
        element.pop('args')
        parser_image.add_argument(*args, **element)

    # set parser_image
    parser_image.set_defaults(handler=run_subcommand, subcommand="image")

    # suggest
    # ----------
    parser_suggest = subparsers.add_parser(
        'suggest',
        help='suggest mode. see `suggest -h`'
    )

    # add_argument
    for element in suggest_args_map:
        args = element['args']
        element.pop('args')
        parser_suggest.add_argument(*args, **element)

    parser_suggest.set_defaults(handler=run_subcommand, subcommand="suggest")

    # --version(-v)オプションのparser定義
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        version='%(prog)s version:{version}'.format(version=__version__)
    )

    args = parser.parse_args()
    if hasattr(args, 'handler'):
        args.handler(args.subcommand, args)
    else:
        # 未知のサブコマンドの場合はヘルプを表示
        parser.print_help()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# =======================================================


import os
import platform

import setuptools
import sphinx.ext.apidoc
from sphinx.setup_command import BuildDoc


class BuildDocApiDoc(BuildDoc, object):
    # inherit from object to enable 'super'
    user_options = []
    description = 'sphinx'

    def run(self):
        # metadata contains information supplied in setup()
        metadata = self.distribution.metadata

        # package_dir may be None, in that case use the current directory.
        src_dir = (self.distribution.package_dir or {'': ''})['']
        src_dir = os.path.join(os.getcwd(), src_dir, 'docs')
        print(src_dir)

        # Run sphinx by calling the main method, '--full' also adds a conf.py
        cmd_line = ['-f', '-H', metadata.name, '-A', metadata.author,
                    '-V', metadata.version, '-R', metadata.version, '-T',
                    '-o', os.path.join('docs', '_build'), src_dir]

        print(cmd_line)
        sphinx.ext.apidoc.main(cmd_line)

        super(BuildDocApiDoc, self).run()


try:
    with open('README.rst') as f:
        readme = f.read()
except IOError:
    readme = ''


# 補完ファイルインストール用関数
def get_data_files():

    # 補完ファイルのインストール先を取得する関数
    def get_completefile_install_location(shell):
        # pathのprefixを定義
        prefix = ''

        # osの種類を取得
        uname = platform.uname()[0]

        # 実行ユーザがrootかどうかでprefixを変更
        if os.geteuid() == 0:
            ''' システムインストール時の挙動 '''
            if uname == 'Linux' and shell == 'bash':
                prefix = '/'
            elif uname == 'Linux' and shell == 'zsh':
                prefix = '/usr/local'
            elif uname == 'Darwin' and shell == 'bash':
                prefix = '/'
            elif uname == 'Darwin' and shell == 'zsh':
                prefix = '/usr'

        # shellの種類に応じてインストール先のlocationを変更
        if shell == 'bash':
            location = os.path.join(prefix, 'etc/bash_completion.d')
        elif shell == 'zsh':
            location = os.path.join(prefix, 'share/zsh/site-functions')
        else:
            raise ValueError('unsupported shell: {0}'.format(shell))

        # locationを返す
        return location

    # locationをdict形式で取得する
    loc = {
        'bash': get_completefile_install_location('bash'),
        'zsh': get_completefile_install_location('zsh')
    }

    # 対象となるファイルをdict形式で指定
    files = dict(
        bash=['completion/pydork-completion.bash'],
        zsh=[
            'completion/pydork-completion.bash',
            'completion/_pydork'
        ]
    )

    # data_files形式でreturn
    data_files = []
    data_files.append((loc['bash'], files['bash']))
    data_files.append((loc['zsh'], files['zsh']))
    return data_files


name = 'pydork'
version = '1.0.0'
release = '1.0.0'

if __name__ == "__main__":
    setuptools.setup(
        name=name,
        version=version,
        author='blacknon',
        author_email='blacknon@orebibou.com',
        maintainer='blacknon',
        maintainer_email='blacknon@orebibou.com',
        description='Scraping and listing text and image searches on Google, Bing, DuckDuckGo, Baidu, Yahoo japan.',
        long_description=readme,
        license='MIT License',
        install_requires=[
            'bs4',
            'chromedriver_autoinstaller',
            'geckodriver_autoinstaller',
            'fake_useragent',
            'lxml',
            'requests[socks]',
            'selenium',
            'selenium_requests',
            'pickle-mixin',
            'sphinx',
        ],
        url='https://github.com/blacknon/pydork',
        packages=setuptools.find_packages(),
        py_modules=['pydork'],
        entry_points={
            'console_scripts': [
                'pydork = pydork:main',
            ],
        },
        classifiers=[
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'License :: OSI Approved :: MIT License',
        ],
        data_files=get_data_files(),
        cmdclass={'build_sphinx': BuildDocApiDoc},
        command_options={
            'build_sphinx': {
                'project': ('setup.py', name),
                'version': ('setup.py', version),
                'release': ('setup.py', release)}},
        setup_requires=['sphinx'],
    )
